import re
from datetime import timedelta

from django.contrib.auth import get_user_model
from django.contrib.auth.forms import PasswordChangeForm
from django.core import mail
from django.test import TestCase
from django.urls import reverse
from django.utils import timezone

from userena import forms
from userena import settings as userena_settings
from userena.models import UserenaSignup
from userena.utils import get_user_profile

User = get_user_model()


class UserenaViewsTests(TestCase):
    """ Test the account views """

    fixtures = ["users", "profiles"]

    def test_valid_activation(self):
        """ A ``POST`` to the activation view """
        # First, register an account.
        self.client.post(
            reverse("userena_signup"),
            data={
                "username": "alice",
                "email": "alice@example.com",
                "password1": "swordfish",
                "password2": "swordfish",
                "tos": "on",
            },
        )
        user = User.objects.get(email="alice@example.com")

        response = self.client.get(
            reverse(
                "userena_activate",
                kwargs={"activation_key": user.userena_signup.activation_key},
            )
        )
        self.assertContains(response, "Activate your account")

        response = self.client.post(
            reverse(
                "userena_activate",
                kwargs={"activation_key": user.userena_signup.activation_key},
            )
        )
        self.assertRedirects(
            response,
            reverse("userena_profile_detail", kwargs={"username": user.username}),
        )

        user = User.objects.get(email="alice@example.com")
        self.assertTrue(user.is_active)

    def test_activation_expired_retry(self):
        """ A ``GET`` to the activation view when activation link is expired """
        # First, register an account.
        userena_settings.USERENA_ACTIVATION_RETRY = True
        self.client.post(
            reverse("userena_signup"),
            data={
                "username": "alice",
                "email": "alice@example.com",
                "password1": "swordfish",
                "password2": "swordfish",
                "tos": "on",
            },
        )
        user = User.objects.get(email="alice@example.com")
        user.date_joined = timezone.now() - timedelta(days=30)
        user.save()
        response = self.client.post(
            reverse(
                "userena_activate",
                kwargs={"activation_key": user.userena_signup.activation_key},
            )
        )
        self.assertContains(response, "Request a new activation link")

        user = User.objects.get(email="alice@example.com")
        self.assertTrue(not user.is_active)
        userena_settings.USERENA_ACTIVATION_RETRY = False

    def test_retry_activation_ask(self):
        """ Ask for a new activation link """
        # First, register an account.
        userena_settings.USERENA_ACTIVATION_RETRY = True
        self.client.post(
            reverse("userena_signup"),
            data={
                "username": "alice",
                "email": "alice@example.com",
                "password1": "swordfish",
                "password2": "swordfish",
                "tos": "on",
            },
        )
        user = User.objects.get(email="alice@example.com")
        user.date_joined = timezone.now() - timedelta(days=30)
        user.save()
        old_key = user.userena_signup.activation_key
        response = self.client.get(
            reverse("userena_activate_retry", kwargs={"activation_key": old_key})
        )

        # We must reload the object from database to get the new key
        user = User.objects.get(email="alice@example.com")
        self.assertContains(response, "Account re-activation succeeded")

        self.assertNotEqual(old_key, user.userena_signup.activation_key)
        user = User.objects.get(email="alice@example.com")
        self.assertTrue(not user.is_active)

        self.assertEqual(len(mail.outbox), 2)
        self.assertEqual(mail.outbox[1].to, ["alice@example.com"])
        self.assertTrue(mail.outbox[1].body.find("activate your account ") > -1)

        response = self.client.post(
            reverse(
                "userena_activate",
                kwargs={"activation_key": user.userena_signup.activation_key},
            )
        )
        self.assertRedirects(
            response,
            reverse("userena_profile_detail", kwargs={"username": user.username}),
        )

        user = User.objects.get(email="alice@example.com")
        self.assertTrue(user.is_active)
        userena_settings.USERENA_ACTIVATION_RETRY = False

    def test_invalid_activation(self):
        """
        A ``GET`` to the activation view with a wrong ``activation_key``.

        """
        response = self.client.post(
            reverse("userena_activate", kwargs={"activation_key": "fake"})
        )
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "userena/activate_fail.html")

    def test_valid_confirmation(self):
        """ A ``GET`` to the verification view """
        # First, try to change an email.
        user = User.objects.get(pk=1)
        user.userena_signup.change_email("johnie@example.com")

        response = self.client.get(
            reverse(
                "userena_email_confirm",
                kwargs={"confirmation_key": user.userena_signup.email_confirmation_key},
            )
        )

        self.assertRedirects(
            response,
            reverse(
                "userena_email_confirm_complete", kwargs={"username": user.username}
            ),
        )

    def test_invalid_confirmation(self):
        """
        A ``GET`` to the verification view with an invalid verification key.

        """
        response = self.client.get(
            reverse("userena_email_confirm", kwargs={"confirmation_key": "WRONG"})
        )
        self.assertTemplateUsed(response, "userena/email_confirm_fail.html")

    def test_disabled_view(self):
        """ A ``GET`` to the ``disabled`` view """
        response = self.client.get(
            reverse("userena_disabled", kwargs={"username": "john"})
        )
        self.assertEqual(response.status_code, 404)

        User.objects.filter(username="john").update(is_active=False)
        response = self.client.get(
            reverse("userena_disabled", kwargs={"username": "john"})
        )
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "userena/disabled.html")

        # Add an activation key to simulate that the account has not been
        # activated yet
        signup = UserenaSignup.objects.get(user__username="john")
        signup.activation_key = "foo"
        signup.save()

        response = self.client.get(
            reverse("userena_disabled", kwargs={"username": "john"})
        )
        self.assertRedirects(
            response, reverse("userena_activate_pending", kwargs={"username": "john"}),
        )

    def test_signup_view(self):
        """ A ``GET`` to the ``signup`` view """
        response = self.client.get(reverse("userena_signup"))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "userena/signup_form.html")

        # Check that the correct form is used.
        self.assertTrue(isinstance(response.context["form"], forms.SignupForm))

        # Now check that a different form is used when
        # ``USERENA_WITHOUT_USERNAMES`` setting is set to ``True``
        userena_settings.USERENA_WITHOUT_USERNAMES = True

        response = self.client.get(reverse("userena_signup"))
        self.assertTrue(isinstance(response.context["form"], forms.SignupFormOnlyEmail))

        # Back to default
        userena_settings.USERENA_WITHOUT_USERNAMES = False

        # Check for 403 with signups disabled
        userena_settings.USERENA_DISABLE_SIGNUP = True

        response = self.client.get(reverse("userena_signup"))
        self.assertEqual(response.status_code, 403)

        # Back to default
        userena_settings.USERENA_DISABLE_SIGNUP = False

    def test_signup_view_signout(self):
        """ Check that a newly signed user shouldn't be signed in. """
        # User should be signed in
        self.assertTrue(self.client.login(username="john", password="blowfish"))
        # Post a new, valid signup
        response = self.client.post(
            reverse("userena_signup"),
            data={
                "username": "alice",
                "email": "alice@example.com",
                "password1": "blueberry",
                "password2": "blueberry",
                "tos": "on",
            },
        )

        # And should now be signed out
        self.assertFalse(len(self.client.session.keys()) > 0)

    def test_signup_view_success(self):
        """
        After a ``POST`` to the ``signup`` view a new user should be created,
        the user should be logged in and redirected to the signup success page.

        """
        response = self.client.post(
            reverse("userena_signup"),
            data={
                "username": "alice",
                "email": "alice@example.com",
                "password1": "blueberry",
                "password2": "blueberry",
                "tos": "on",
            },
        )

        # Check for redirect.
        self.assertRedirects(
            response, reverse("userena_signup_complete", kwargs={"username": "alice"})
        )

        # Check for new user.
        self.assertEqual(
            User.objects.filter(email__iexact="alice@example.com").count(), 1
        )

    def test_signup_view_with_signin(self):
        """
        After a ``POST`` to the ``signup`` view a new user should be created,
        the user should be logged in and redirected to the signup success page.

        """
        # If activation is required, user is not logged in after signup,
        # disregarding USERENA_SIGNIN_AFTER_SIGNUP setting
        userena_settings.USERENA_SIGNIN_AFTER_SIGNUP = True
        userena_settings.USERENA_ACTIVATION_REQUIRED = True
        response = self.client.post(
            reverse("userena_signup"),
            data={
                "username": "alice",
                "email": "alice@example.com",
                "password1": "blueberry",
                "password2": "blueberry",
                "tos": "on",
            },
        )
        # Immediate reset to default to avoid leaks
        userena_settings.USERENA_SIGNIN_AFTER_SIGNUP = False
        userena_settings.USERENA_ACTIVATION_REQUIRED = True

        response_check = self.client.get(
            reverse("userena_profile_edit", kwargs={"username": "alice"})
        )
        self.assertEqual(response_check.status_code, 403)

        userena_settings.USERENA_SIGNIN_AFTER_SIGNUP = True
        userena_settings.USERENA_ACTIVATION_REQUIRED = False
        response = self.client.post(
            reverse("userena_signup"),
            data={
                "username": "johndoe",
                "email": "johndoe@example.com",
                "password1": "blueberry",
                "password2": "blueberry",
                "tos": "on",
            },
        )
        # Immediate reset to default to avoid leaks
        userena_settings.USERENA_SIGNIN_AFTER_SIGNUP = False
        userena_settings.USERENA_ACTIVATION_REQUIRED = True

        # Kind of hackish way to check if the user is logged in
        response_check = self.client.get(
            reverse("userena_profile_edit", kwargs={"username": "johndoe"})
        )
        self.assertEqual(response_check.status_code, 200)

    def test_signin_view(self):
        """ A ``GET`` to the signin view should render the correct form """
        response = self.client.get(reverse("userena_signin"))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "userena/signin_form.html")

    def test_signin_view_remember_me_on(self):
        """
        A ``POST`` to the signin with tells it to remember the user for
        ``REMEMBER_ME_DAYS``.

        """
        response = self.client.post(
            reverse("userena_signin"),
            data={
                "identification": "john@example.com",
                "password": "blowfish",
                "remember_me": True,
            },
        )
        self.assertEqual(
            self.client.session.get_expiry_age(),
            userena_settings.USERENA_REMEMBER_ME_DAYS[1] * 3600 * 24,
        )

    def test_signin_view_remember_off(self):
        """
        A ``POST`` to the signin view of which the user doesn't want to be
        remembered.

        """
        response = self.client.post(
            reverse("userena_signin"),
            data={"identification": "john@example.com", "password": "blowfish"},
        )

        self.assertTrue(self.client.session.get_expire_at_browser_close())

    def test_signin_view_inactive(self):
        """ A ``POST`` from a inactive user """
        user = User.objects.get(email="john@example.com")
        user.is_active = False
        user.save()

        response = self.client.post(
            reverse("userena_signin"),
            data={"identification": "john@example.com", "password": "blowfish"},
        )

        self.assertRedirects(
            response, reverse("userena_disabled", kwargs={"username": user.username})
        )

    def test_signin_unactivated(self):
        self.client.post(
            reverse("userena_signup"),
            data={
                "username": "alice",
                "email": "alice@example.com",
                "password1": "swordfish",
                "password2": "swordfish",
                "tos": "on",
            },
        )

        user = User.objects.get(email="alice@example.com")

        response = self.client.post(
            reverse("userena_signin"),
            data={"identification": user.email, "password": "swordfish"},
        )

        self.assertRedirects(
            response,
            reverse("userena_activate_pending", kwargs={"username": user.username}),
        )

    def test_activate_pending_disabled_redirect(self):
        response = self.client.get(
            reverse("userena_activate_pending", kwargs={"username": "john"})
        )
        self.assertEqual(response.status_code, 404)

        User.objects.filter(username="john").update(is_active=False)
        response = self.client.get(
            reverse("userena_activate_pending", kwargs={"username": "john"})
        )
        self.assertRedirects(
            response, reverse("userena_disabled", kwargs={"username": "john"}),
        )

    def test_signin_view_success(self):
        """
        A valid ``POST`` to the signin view should redirect the user to it's
        own profile page if no ``next`` value is supplied. Else it should
        redirect to ``next``.

        """
        response = self.client.post(
            reverse("userena_signin"),
            data={"identification": "john@example.com", "password": "blowfish"},
        )

        self.assertRedirects(
            response, reverse("userena_profile_detail", kwargs={"username": "john"})
        )

        # Redirect to supplied ``next`` value.
        response = self.client.post(
            reverse("userena_signin"),
            data={
                "identification": "john@example.com",
                "password": "blowfish",
                "next": "/accounts/",
            },
        )
        self.assertRedirects(response, "/accounts/")

    def test_signin_view_with_invalid_next(self):
        """
        If the value of "next" is not a real URL, this should not raise
        an exception
        """
        response = self.client.post(
            reverse("userena_signin"),
            data={
                "identification": "john@example.com",
                "password": "blowfish",
                "next": "something-fake",
            },
            follow=True,
        )
        self.assertEqual(response.status_code, 404)

    def test_signout_view(self):
        """ A ``GET`` to the signout view """
        response = self.client.get(reverse("userena_signout"))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "userena/signout.html")

    def test_change_email_view(self):
        """ A ``GET`` to the change e-mail view. """
        response = self.client.get(
            reverse("userena_email_change", kwargs={"username": "john"})
        )

        # Anonymous user should not be able to view the profile page
        self.assertEqual(response.status_code, 403)

        # Login
        client = self.client.login(username="john", password="blowfish")
        response = self.client.get(
            reverse("userena_email_change", kwargs={"username": "john"})
        )

        self.assertEqual(response.status_code, 200)

        # Check that the correct form is used.
        self.assertTrue(isinstance(response.context["form"], forms.ChangeEmailForm))

        self.assertTemplateUsed(response, "userena/email_form.html")

    def test_change_valid_email_view(self):
        """ A ``POST`` with a valid e-mail address """
        self.client.login(username="john", password="blowfish")
        response = self.client.post(
            reverse("userena_email_change", kwargs={"username": "john"}),
            data={"email": "john_new@example.com"},
        )

        self.assertRedirects(
            response,
            reverse("userena_email_change_complete", kwargs={"username": "john"}),
        )

    def test_change_password_view(self):
        """ A ``GET`` to the change password view """
        self.client.login(username="john", password="blowfish")
        response = self.client.get(
            reverse("userena_password_change", kwargs={"username": "john"})
        )

        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "userena/password_form.html")
        self.assertTrue(response.context["form"], PasswordChangeForm)

    def test_change_password_view_success(self):
        """ A valid ``POST`` to the password change view """
        self.client.login(username="john", password="blowfish")

        new_password = "suckfish"
        response = self.client.post(
            reverse("userena_password_change", kwargs={"username": "john"}),
            data={
                "new_password1": new_password,
                "new_password2": "suckfish",
                "old_password": "blowfish",
            },
        )

        self.assertRedirects(
            response,
            reverse("userena_password_change_complete", kwargs={"username": "john"}),
        )

        # Check that the new password is set.
        john = User.objects.get(username="john")
        self.assertTrue(john.check_password(new_password))

    def test_profile_detail_view(self):
        """ A ``GET`` to the detailed view of a user """
        response = self.client.get(
            reverse("userena_profile_detail", kwargs={"username": "john"})
        )

        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "userena/profile_detail.html")

    def test_profile_edit_view(self):
        """ A ``GET`` to the edit view of a users account """
        self.client.login(username="john", password="blowfish")
        response = self.client.get(
            reverse("userena_profile_edit", kwargs={"username": "john"})
        )

        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "userena/profile_form.html")
        self.assertTrue(isinstance(response.context["form"], forms.EditProfileForm))

    def test_profile_edit_view_success(self):
        """ A ``POST`` to the edit view """
        self.client.login(username="john", password="blowfish")
        new_about_me = "I hate it when people use my name for testing."
        response = self.client.post(
            reverse("userena_profile_edit", kwargs={"username": "john"}),
            data={"about_me": new_about_me, "privacy": "open", "language": "en"},
        )

        # A valid post should redirect to the detail page.
        self.assertRedirects(
            response, reverse("userena_profile_detail", kwargs={"username": "john"})
        )

        # Users hould be changed now.
        profile = get_user_profile(user=User.objects.get(username="john"))
        self.assertEqual(profile.about_me, new_about_me)

    def test_profile_list_view(self):
        """ A ``GET`` to the list view of a user """

        # A profile list should be shown.
        userena_settings.USERENA_DISABLE_PROFILE_LIST = False
        response = self.client.get(reverse("userena_profile_list"))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "userena/profile_list.html")

        # Profile list is disabled.
        userena_settings.USERENA_DISABLE_PROFILE_LIST = True
        response = self.client.get(reverse("userena_profile_list"))
        self.assertEqual(response.status_code, 404)

    def test_password_reset_view_success(self):
        """ A ``POST`` to the password reset view with email that exists"""
        response = self.client.post(
            reverse("userena_password_reset"), data={"email": "john@example.com"}
        )
        # check if there was success redirect to userena_password_reset_done
        # and email was sent
        self.assertEqual(response.status_code, 302)
        self.assertIn(reverse("userena_password_reset_done"), response.get("Location"))
        self.assertTrue(mail.outbox)

    def test_password_reset_view_failure(self):
        """ A ``POST`` to the password reset view with incorrect email"""
        response = self.client.post(
            reverse("userena_password_reset"),
            data={"email": "no.such.user@example.com"},
        )
        # note: status code can be different depending on django version
        self.assertIn(response.status_code, [200, 302])
        self.assertFalse(mail.outbox)

    def test_password_reset_confirm(self):
        # post reset request and search form confirmation url
        self.client.post(
            reverse("userena_password_reset"), data={"email": "john@example.com"}
        )
        confirm_mail = mail.outbox[0]
        confirm_url = re.search(r"\bhttps?://\S+", confirm_mail.body).group()

        # get confirmation request page
        response = self.client.get(confirm_url)
        self.assertEqual(response.status_code, 302)

        confirm_url = response.url

        # post new password and check if redirected with success
        response = self.client.post(
            confirm_url, data={"new_password1": "pass", "new_password2": "pass"}
        )
        self.assertEqual(response.status_code, 302)
        self.assertIn(
            reverse("userena_password_reset_complete"), response.get("Location")
        )



from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [migrations.swappable_dependency(settings.AUTH_USER_MODEL)]

    operations = [
        migrations.CreateModel(
            name="UserenaSignup",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                        primary_key=True,
                    ),
                ),
                (
                    "last_active",
                    models.DateTimeField(
                        help_text="The last date that the user was active.",
                        null=True,
                        verbose_name="last active",
                        blank=True,
                    ),
                ),
                (
                    "activation_key",
                    models.CharField(
                        max_length=40, verbose_name="activation key", blank=True
                    ),
                ),
                (
                    "activation_notification_send",
                    models.BooleanField(
                        default=False,
                        help_text="Designates whether this user has already got a notification about activating their account.",
                        verbose_name="notification send",
                    ),
                ),
                (
                    "email_unconfirmed",
                    models.EmailField(
                        help_text="Temporary email address when the user requests an email change.",
                        max_length=75,
                        verbose_name="unconfirmed email address",
                        blank=True,
                    ),
                ),
                (
                    "email_confirmation_key",
                    models.CharField(
                        max_length=40,
                        verbose_name="unconfirmed email verification key",
                        blank=True,
                    ),
                ),
                (
                    "email_confirmation_key_created",
                    models.DateTimeField(
                        null=True,
                        verbose_name="creation date of email confirmation key",
                        blank=True,
                    ),
                ),
                (
                    "user",
                    models.OneToOneField(
                        related_name="userena_signup",
                        verbose_name="user",
                        to=settings.AUTH_USER_MODEL,
                        on_delete=models.CASCADE,
                    ),
                ),
            ],
            options={
                "verbose_name": "userena registration",
                "verbose_name_plural": "userena registrations",
            },
            bases=(models.Model,),
        )
    ]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['userena',
 'userena.contrib',
 'userena.contrib.umessages',
 'userena.contrib.umessages.migrations',
 'userena.contrib.umessages.south_migrations',
 'userena.contrib.umessages.templatetags',
 'userena.contrib.umessages.tests',
 'userena.management',
 'userena.management.commands',
 'userena.migrations',
 'userena.runtests',
 'userena.tests',
 'userena.tests.profiles']

package_data = \
{'': ['*'],
 'userena': ['fixtures/*',
             'locale/ar/LC_MESSAGES/*',
             'locale/bg/LC_MESSAGES/*',
             'locale/de/LC_MESSAGES/*',
             'locale/de_du/LC_MESSAGES/*',
             'locale/el/LC_MESSAGES/*',
             'locale/es/LC_MESSAGES/*',
             'locale/fr/LC_MESSAGES/*',
             'locale/gl/LC_MESSAGES/*',
             'locale/it/LC_MESSAGES/*',
             'locale/nb/LC_MESSAGES/*',
             'locale/nl/LC_MESSAGES/*',
             'locale/pl/LC_MESSAGES/*',
             'locale/pt/LC_MESSAGES/*',
             'locale/pt_BR/LC_MESSAGES/*',
             'locale/ro/LC_MESSAGES/*',
             'locale/ru/LC_MESSAGES/*',
             'locale/tr/LC_MESSAGES/*',
             'locale/zh_Hans/LC_MESSAGES/*',
             'locale/zh_Hant/LC_MESSAGES/*',
             'templates/*',
             'templates/userena/*',
             'templates/userena/emails/*'],
 'userena.contrib.umessages': ['fixtures/*',
                               'templates/*',
                               'templates/umessages/*'],
 'userena.runtests': ['private/*', 'templates/*']}

install_requires = \
['Django>=3.2,<5.0', 'django-guardian>=2.0', 'easy-thumbnails', 'html2text']

setup_kwargs = {
    'name': 'django-userena-ce',
    'version': '7.0.0a1',
    'description': 'Complete user management application for Django',
    'long_description': "# Django Userena (Community Edition)\n\n[![Build Status](https://github.com/django-userena-ce/django-userena-ce/workflows/CI/badge.svg)](https://github.com/django-userena-ce/django-userena-ce/actions?query=workflow%3ACI+branch%3Amaster)\n[![PyPI version](https://badge.fury.io/py/django-userena-ce.svg)](https://badge.fury.io/py/django-userena-ce)\n[![PyPI - Python Version](https://img.shields.io/pypi/pyversions/django-userena-ce)](https://pypi.org/project/django-userena-ce/)\n[![Coverage Status](https://coveralls.io/repos/github/django-userena-ce/django-userena-ce/badge.svg?branch=master)](https://coveralls.io/github/django-userena-ce/django-userena-ce?branch=master)\n[![Documentation Status](https://readthedocs.org/projects/django-userena-ce/badge/?version=latest)](http://django-userena-ce.readthedocs.io/en/latest/?badge=latest)\n\nThis project is a community edition fork of\n[django-userena](https://github.com/bread-and-pepper/django-userena).\n\nUserena is a Django application that supplies your Django project with full\naccount management. It's a fully customizable application that takes care of\nthe signup, activation, messaging and more. It's BSD licensed, which means you\ncan use it commercially for free!\n\n## Supported Software\n\nSoftware | Versions\n---|---\nPython | 3.7, 3.8, 3.9, 3.10\nDjango | 3.2, 4.0\nDjango Guardian | \\>2.0\n\n## [Documentation](https://django-userena-ce.readthedocs.io/en/latest/index.html)\n\nComplete documentation about the\n[installation](https://django-userena-ce.readthedocs.io/en/latest/installation.html),\n[settings](https://django-userena-ce.readthedocs.io/en/latest/settings.html) and\n[F.A.Q.](https://django-userena-ce.readthedocs.io/en/latest/faq.html) is available on\n[Read the Docs](https://django-userena-ce.readthedocs.io/en/latest/index.html).\n\nFor list of updates and changes see `UPDATES.md` file.\n",
    'author': 'James Meakin',
    'author_email': 'django-userena-ce@jmsmkn.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/django-userena-ce/django-userena-ce',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<3.11',
}


setup(**setup_kwargs)



from django.db import models
from django.db.models import Q
from django.core.exceptions import ObjectDoesNotExist
from django.contrib.auth import get_user_model
from django.contrib.auth.models import UserManager, Permission, AnonymousUser
from django.contrib.contenttypes.models import ContentType
from django.utils.encoding import smart_str
from django.utils.translation import gettext as _
from django.conf import settings

from userena import settings as userena_settings
from userena.utils import (
    generate_nonce,
    get_profile_model,
    get_datetime_now,
    get_user_profile,
)
from userena import signals as userena_signals

from guardian.shortcuts import assign_perm, get_perms


import re

NONCE_RE = re.compile(r"^[\w]{40}$")

ASSIGNED_PERMISSIONS = {
    "profile": (
        ("view_profile", "Can view profile"),
        ("change_profile", "Can change profile"),
        ("delete_profile", "Can delete profile"),
    ),
    "user": (("change_user", "Can change user"), ("delete_user", "Can delete user")),
}


class UserenaManager(UserManager):
    """ Extra functionality for the Userena model. """

    def create_user(self, username, email, password, active=False, send_email=True):
        """
        A simple wrapper that creates a new :class:`User`.

        :param username:
            String containing the username of the new user.

        :param email:
            String containing the email address of the new user.

        :param password:
            String containing the password for the new user.

        :param active:
            Boolean that defines if the user requires activation by clicking
            on a link in an e-mail. Defaults to ``False``.

        :param send_email:
            Boolean that defines if the user should be sent an email. You could
            set this to ``False`` when you want to create a user in your own
            code, but don't want the user to activate through email.

        :return: :class:`User` instance representing the new user.

        """

        new_user = get_user_model().objects.create_user(username, email, password)
        new_user.is_active = active
        new_user.save()

        # Give permissions to view and change profile
        for perm in ASSIGNED_PERMISSIONS["profile"]:
            assign_perm(perm[0], new_user, get_user_profile(user=new_user))

        # Give permissions to view and change itself
        for perm in ASSIGNED_PERMISSIONS["user"]:
            assign_perm(perm[0], new_user, new_user)

        userena_profile = self.create_userena_profile(new_user)

        if send_email:
            userena_profile.send_activation_email()

        return new_user

    def create_userena_profile(self, user):
        """
        Creates an :class:`UserenaSignup` instance for this user.

        :param user:
            Django :class:`User` instance.

        :return: The newly created :class:`UserenaSignup` instance.

        """
        if isinstance(user.username, str):
            user.username = smart_str(user.username)

        try:
            profile = self.get(user=user)
        except self.model.DoesNotExist:
            profile = self.create(user=user, activation_key=generate_nonce())
        return profile

    def reissue_activation(self, activation_key):
        """
        Creates a new ``activation_key`` resetting activation timeframe when
        users let the previous key expire.

        :param activation_key:
            String containing the secret nonce activation key.

        """
        try:
            userena = self.get(activation_key=activation_key)
        except self.model.DoesNotExist:
            return False
        try:
            userena.activation_key = generate_nonce()
            userena.save(using=self._db)
            userena.user.date_joined = get_datetime_now()
            userena.user.save(using=self._db)
            userena.send_activation_email()
            return True
        except Exception:
            return False

    def activate_user(self, activation_key):
        """
        Activate an :class:`User` by supplying a valid ``activation_key``.

        If the key is valid and an user is found, activates the user and
        return it. Also sends the ``activation_complete`` signal.

        :param activation_key:
            String containing the secret nonce for a valid activation.

        :return:
            The newly activated :class:`User` or ``False`` if not successful.

        """
        if NONCE_RE.search(activation_key):
            try:
                userena = self.get(activation_key=activation_key)
            except self.model.DoesNotExist:
                return False
            if not userena.activation_key_expired():
                userena.activation_key = userena_settings.USERENA_ACTIVATED
                user = userena.user
                user.is_active = True
                userena.save(using=self._db)
                user.save(using=self._db)

                # Send the activation_complete signal
                userena_signals.activation_complete.send(sender=None, user=user)

                return user
        return False

    def check_expired_activation(self, activation_key):
        """
        Check if ``activation_key`` is still valid.

        Raises a ``self.model.DoesNotExist`` exception if key is not present or
         ``activation_key`` is not a valid string

        :param activation_key:
            String containing the secret nonce for a valid activation.

        :return:
            True if the ket has expired, False if still valid.

        """
        if NONCE_RE.search(activation_key):
            userena = self.get(activation_key=activation_key)
            return userena.activation_key_expired()
        raise self.model.DoesNotExist

    def confirm_email(self, confirmation_key):
        """
        Confirm an email address by checking a ``confirmation_key``.

        A valid ``confirmation_key`` will set the newly wanted e-mail
        address as the current e-mail address. Returns the user after
        success or ``False`` when the confirmation key is
        invalid. Also sends the ``confirmation_complete`` signal.

        :param confirmation_key:
            String containing the secret nonce that is used for verification.

        :return:
            The verified :class:`User` or ``False`` if not successful.

        """
        if NONCE_RE.search(confirmation_key):
            try:
                userena = self.get(
                    email_confirmation_key=confirmation_key,
                    email_unconfirmed__isnull=False,
                )
            except self.model.DoesNotExist:
                return False
            else:
                user = userena.user
                old_email = user.email
                user.email = userena.email_unconfirmed
                userena.email_unconfirmed, userena.email_confirmation_key = "", ""
                userena.save(using=self._db)
                user.save(using=self._db)

                # Send the confirmation_complete signal
                userena_signals.confirmation_complete.send(
                    sender=None, user=user, old_email=old_email
                )

                return user
        return False

    def delete_expired_users(self):
        """
        Checks for expired users and delete's the ``User`` associated with
        it. Skips if the user ``is_staff``.

        :return: A list containing the deleted users.

        """
        deleted_users = []
        for user in get_user_model().objects.filter(is_staff=False, is_active=False):
            if user.userena_signup.activation_key_expired():
                deleted_users.append(user)
                user.delete()
        return deleted_users

    def check_permissions(self):
        """
        Checks that all permissions are set correctly for the users.

        :return: A set of users whose permissions was wrong.

        """
        # Variable to supply some feedback
        changed_permissions = []
        changed_users = []
        warnings = []

        # Check that all the permissions are available.
        for model, perms in ASSIGNED_PERMISSIONS.items():
            if model == "profile":
                model_obj = get_profile_model()
            else:
                model_obj = get_user_model()

            model_content_type = ContentType.objects.get_for_model(model_obj)

            for perm in perms:
                try:
                    Permission.objects.get(
                        codename=perm[0], content_type=model_content_type
                    )
                except Permission.DoesNotExist:
                    changed_permissions.append(perm[1])
                    Permission.objects.create(
                        name=perm[1], codename=perm[0], content_type=model_content_type
                    )

        # it is safe to rely on settings.ANONYMOUS_USER_NAME since it is a
        # requirement of django-guardian
        for user in get_user_model().objects.exclude(
            username=settings.ANONYMOUS_USER_NAME
        ):
            try:
                user_profile = get_user_profile(user=user)
            except ObjectDoesNotExist:
                warnings.append(
                    _("No profile found for %(username)s") % {"username": user.username}
                )
            else:
                all_permissions = get_perms(user, user_profile) + get_perms(user, user)

                for model, perms in ASSIGNED_PERMISSIONS.items():
                    if model == "profile":
                        perm_object = get_user_profile(user=user)
                    else:
                        perm_object = user

                    for perm in perms:
                        if perm[0] not in all_permissions:
                            assign_perm(perm[0], user, perm_object)
                            changed_users.append(user)

        return (changed_permissions, changed_users, warnings)


class UserenaBaseProfileManager(models.Manager):
    """ Manager for :class:`UserenaProfile` """

    def get_visible_profiles(self, user=None):
        """
        Returns all the visible profiles available to this user.

        For now keeps it simple by just applying the cases when a user is not
        active, a user has it's profile closed to everyone or a user only
        allows registered users to view their profile.

        :param user:
            A Django :class:`User` instance.

        :return:
            All profiles that are visible to this user.

        """
        profiles = self.all()

        filter_kwargs = {"user__is_active": True}

        profiles = profiles.filter(**filter_kwargs)
        if user and isinstance(user, AnonymousUser):
            profiles = profiles.exclude(Q(privacy="closed") | Q(privacy="registered"))
        else:
            profiles = profiles.exclude(Q(privacy="closed"))
        return profiles



# Django settings for Userena demo project.
DEBUG = True

import os

settings_dir = os.path.dirname(__file__)
PROJECT_ROOT = os.path.abspath(settings_dir)

ADMINS = (
    # ('Your Name', 'your_email@example.com'),
)

MANAGERS = ADMINS

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": os.path.join(PROJECT_ROOT, "private/development.db"),
    }
}

# Internationalization
TIME_ZONE = "America/Chicago"
LANGUAGE_CODE = "en-us"
gettext = lambda s: s
LANGUAGES = (
    ("en", gettext("English")),
    ("nl", gettext("Dutch")),
    ("fr", gettext("French")),
    ("pl", gettext("Polish")),
    ("pt", gettext("Portugese")),
    ("pt-br", gettext("Brazilian Portuguese")),
    ("es", gettext("Spanish")),
    ("el", gettext("Greek")),
    ("zh-hans", gettext("Simplified Chinese")),
    ("zh-hant", gettext("Traditional Chinese")),
)
LOCALE_PATHS = (os.path.join(PROJECT_ROOT, "locale"),)

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

# If you set this to False, Django will not format dates, numbers and
# calendars according to the current locale.
USE_L10N = True

# If you set this to False, Django will not use timezone-aware datetimes.
USE_TZ = True
MEDIA_ROOT = os.path.join(PROJECT_ROOT, "public/media/")
MEDIA_URL = "/media/"
STATIC_ROOT = os.path.join(PROJECT_ROOT, "public/static/")
STATIC_URL = "/static/"

STATICFILES_DIRS = (os.path.join(PROJECT_ROOT, "demo/static/"),)

# List of finder classes that know how to find static files in
# various locations.
STATICFILES_FINDERS = (
    "django.contrib.staticfiles.finders.FileSystemFinder",
    "django.contrib.staticfiles.finders.AppDirectoriesFinder",
)

# Make this unique, and don't share it with anybody.
SECRET_KEY = "_g-js)o8z#8=9pr1&amp;05h^1_#)91sbo-)g^(*=-+epxmt4kc9m#"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [
            # insert your TEMPLATE_DIRS here
        ],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                # Insert your TEMPLATE_CONTEXT_PROCESSORS here or use this
                # list if you haven't customized them:
                "django.contrib.auth.context_processors.auth",
                "django.template.context_processors.debug",
                "django.template.context_processors.i18n",
                "django.template.context_processors.media",
                "django.template.context_processors.static",
                "django.template.context_processors.tz",
                "django.contrib.messages.context_processors.messages",
            ]
        },
    }
]

MIDDLEWARE = (
    "django.middleware.common.CommonMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "django.middleware.locale.LocaleMiddleware",
    "userena.middleware.UserenaLocaleMiddleware",
)

# Add the Guardian and userena authentication backends
AUTHENTICATION_BACKENDS = (
    "userena.backends.UserenaAuthenticationBackend",
    "guardian.backends.ObjectPermissionBackend",
    "django.contrib.auth.backends.ModelBackend",
)

# Settings used by Userena
LOGIN_REDIRECT_URL = "/accounts/%(username)s/"
LOGIN_URL = "/accounts/signin/"
LOGOUT_URL = "/accounts/signout/"
AUTH_PROFILE_MODULE = "profiles.Profile"
USERENA_DISABLE_PROFILE_LIST = True
USERENA_MUGSHOT_SIZE = 140

ROOT_URLCONF = "userena.runtests.urls"
WSGI_APPLICATION = "demo.wsgi.application"

INSTALLED_APPS = (
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.sites",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "django.contrib.admin",
    "django.contrib.admindocs",
    "guardian",
    "userena",
    "userena.contrib.umessages",
    "userena.tests.profiles",
)

INSTALLED_APPS += ("easy_thumbnails",)

LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "filters": {"require_debug_false": {"()": "django.utils.log.RequireDebugFalse"}},
    "handlers": {
        "mail_admins": {
            "level": "ERROR",
            "filters": ["require_debug_false"],
            "class": "django.utils.log.AdminEmailHandler",
        }
    },
    "loggers": {
        "django.request": {
            "handlers": ["mail_admins"],
            "level": "ERROR",
            "propagate": True,
        }
    },
}

ANONYMOUS_USER_NAME = "AnonymousUser"

USERENA_USE_HTTPS = False



from django.test import TestCase
from django.contrib.auth import get_user_model

from userena.backends import UserenaAuthenticationBackend

User = get_user_model()


class UserenaAuthenticationBackendTests(TestCase):
    """
    Test the ``UserenaAuthenticationBackend`` which should return a ``User``
    when supplied with a username/email and a correct password.

    """

    fixtures = ["users"]
    backend = UserenaAuthenticationBackend()

    def test_with_username(self):
        """ Test the backend when usernames are supplied. """
        # Invalid usernames or passwords
        invalid_data_dicts = [
            # Invalid password
            {"identification": "john", "password": "inhalefish"},
            # Invalid username
            {"identification": "alice", "password": "blowfish"},
        ]
        for invalid_dict in invalid_data_dicts:
            result = self.backend.authenticate(
                request=None,
                identification=invalid_dict["identification"],
                password=invalid_dict["password"],
            )
            self.assertFalse(isinstance(result, User))

        # Valid username and password
        result = self.backend.authenticate(
            request=None, identification="john", password="blowfish"
        )
        self.assertTrue(isinstance(result, User))

    def test_with_email(self):
        """ Test the backend when email address is supplied """
        # Invalid e-mail adressses or passwords
        invalid_data_dicts = [
            # Invalid password
            {"identification": "john@example.com", "password": "inhalefish"},
            # Invalid e-mail address
            {"identification": "alice@example.com", "password": "blowfish"},
        ]
        for invalid_dict in invalid_data_dicts:
            result = self.backend.authenticate(
                request=None,
                identification=invalid_dict["identification"],
                password=invalid_dict["password"],
            )
            self.assertFalse(isinstance(result, User))

        # Valid e-email address and password
        result = self.backend.authenticate(
            request=None, identification="john@example.com", password="blowfish"
        )
        self.assertTrue(isinstance(result, User))

    def test_get_user(self):
        """ Test that the user is returned """
        user = self.backend.get_user(1)
        self.assertEqual(user.username, "john")

        # None should be returned when false id.
        user = self.backend.get_user(99)
        self.assertFalse(user)



import datetime
import hashlib
import re
from urllib.parse import urlparse, parse_qs

from django.contrib.auth import get_user_model
from django.contrib.auth.models import AnonymousUser
from django.core import mail
from django.conf import settings
from django.test import TestCase

from userena.models import UserenaSignup, upload_to_mugshot
from userena import settings as userena_settings
from userena.tests.profiles.models import Profile
from userena.utils import get_user_profile

User = get_user_model()

MUGSHOT_RE = re.compile("^[a-f0-9]{40}$")


class UserenaSignupModelTests(TestCase):
    """ Test the model of UserenaSignup """

    user_info = {
        "username": "alice",
        "password": "swordfish",
        "email": "alice@example.com",
    }

    fixtures = ["users", "profiles"]

    def test_upload_mugshot(self):
        """
        Test the uploaded path of mugshots

        TODO: What if a image get's uploaded with no extension?

        """
        user = User.objects.get(pk=1)
        filename = "my_avatar.png"
        path = upload_to_mugshot(get_user_profile(user=user), filename)

        # Path should be changed from the original
        self.assertNotEqual(filename, path)

        # Check if the correct path is returned
        MUGSHOT_RE = re.compile(
            r"^%(mugshot_path)s[\w]{10}.png$"
            % {"mugshot_path": userena_settings.USERENA_MUGSHOT_PATH}
        )

        self.assertTrue(MUGSHOT_RE.search(path))

    def test_stringification(self):
        """
        Test the stringification of a ``UserenaSignup`` object. A
        "human-readable" representation of an ``UserenaSignup`` object.

        """
        signup = UserenaSignup.objects.get(pk=1)
        self.assertEqual(signup.__str__(), signup.user.username)

    def test_change_email(self):
        """ TODO """
        pass

    def test_activation_expired_account(self):
        """
        ``UserenaSignup.activation_key_expired()`` is ``True`` when the
        ``activation_key_created`` is more days ago than defined in
        ``USERENA_ACTIVATION_DAYS``.

        """
        user = UserenaSignup.objects.create_user(**self.user_info)
        user.date_joined -= datetime.timedelta(
            days=userena_settings.USERENA_ACTIVATION_DAYS + 1
        )
        user.save()

        user = User.objects.get(username="alice")
        self.assertTrue(user.userena_signup.activation_key_expired())

    def test_activation_used_account(self):
        """
        An user cannot be activated anymore once the activation key is
        already used.

        """
        user = UserenaSignup.objects.create_user(**self.user_info)
        activated_user = UserenaSignup.objects.activate_user(
            user.userena_signup.activation_key
        )
        self.assertTrue(activated_user.userena_signup.activation_key_expired())

    def test_activation_unexpired_account(self):
        """
        ``UserenaSignup.activation_key_expired()`` is ``False`` when the
        ``activation_key_created`` is within the defined timeframe.``

        """
        user = UserenaSignup.objects.create_user(**self.user_info)
        self.assertFalse(user.userena_signup.activation_key_expired())

    def test_activation_email(self):
        """
        When a new account is created, a activation e-mail should be send out
        by ``UserenaSignup.send_activation_email``.

        """
        new_user = UserenaSignup.objects.create_user(**self.user_info)
        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].to, [self.user_info["email"]])

    def test_plain_email(self):
        """
        If HTML emails are disabled, check that outgoing emails are not multipart
        """
        userena_settings.USERENA_HTML_EMAIL = False
        new_user = UserenaSignup.objects.create_user(**self.user_info)
        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(
            str(mail.outbox[0].message()).find("multipart/alternative"), -1
        )

    def test_html_email(self):
        """
        If HTML emails are enabled, check outgoings emails are multipart and
        that different html and plain text templates are used
        """
        userena_settings.USERENA_HTML_EMAIL = True
        userena_settings.USERENA_USE_PLAIN_TEMPLATE = True

        new_user = UserenaSignup.objects.create_user(**self.user_info)

        # Reset configuration
        userena_settings.USERENA_HTML_EMAIL = False
        self.assertEqual(len(mail.outbox), 1)
        self.assertTrue(
            str(mail.outbox[0].message()).find("multipart/alternative") > -1
        )
        self.assertTrue(str(mail.outbox[0].message()).find("text/plain") > -1)
        self.assertTrue(str(mail.outbox[0].message()).find("text/html") > -1)
        self.assertTrue(str(mail.outbox[0].message()).find("<html>") > -1)
        self.assertTrue(
            str(mail.outbox[0].message()).find("<p>Thank you for signing up") > -1
        )
        self.assertFalse(mail.outbox[0].body.find("<p>Thank you for signing up") > -1)

    def test_generated_plain_email(self):
        """
        If HTML emails are enabled and plain text template are disabled,
        check outgoings emails are multipart and that plain text is generated
        from html body
        """
        userena_settings.USERENA_HTML_EMAIL = True
        userena_settings.USERENA_USE_PLAIN_TEMPLATE = False

        new_user = UserenaSignup.objects.create_user(**self.user_info)

        # Reset configuration
        userena_settings.USERENA_HTML_EMAIL = False
        userena_settings.USERENA_USE_PLAIN_TEMPLATE = True

        self.assertEqual(len(mail.outbox), 1)
        self.assertTrue(
            str(mail.outbox[0].message()).find("multipart/alternative") > -1
        )
        self.assertTrue(str(mail.outbox[0].message()).find("text/plain") > -1)
        self.assertTrue(str(mail.outbox[0].message()).find("text/html") > -1)
        self.assertTrue(str(mail.outbox[0].message()).find("<html>") > -1)
        self.assertTrue(
            str(mail.outbox[0].message()).find("<p>Thank you for signing up") > -1
        )
        self.assertTrue(mail.outbox[0].body.find("Thank you for signing up") > -1)


class BaseProfileModelTest(TestCase):
    """ Test the ``BaseProfile`` model """

    fixtures = ["users", "profiles"]

    def test_mugshot_url(self):
        """ The user has uploaded it's own mugshot. This should be returned. """
        profile = Profile.objects.get(pk=1)
        profile.mugshot = "fake_image.png"
        profile.save()

        profile = Profile.objects.get(pk=1)
        self.assertEqual(
            profile.get_mugshot_url(), settings.MEDIA_URL + "fake_image.png"
        )

    def test_stringification(self):
        """ Profile should return a human-readable name as an object """
        profile = Profile.objects.get(pk=1)
        self.assertEqual(profile.__str__(), "Profile of %s" % profile.user.username)

    def test_get_mugshot_url_without_gravatar(self):
        """
        Test if the correct mugshot is returned for the user when
        ``USERENA_MUGSHOT_GRAVATAR`` is set to ``False``.

        """
        # This user has no mugshot, and gravatar is disabled. And to make
        # matters worse, there isn't even a default image.
        userena_settings.USERENA_MUGSHOT_GRAVATAR = False
        profile = Profile.objects.get(pk=1)
        self.assertEqual(profile.get_mugshot_url(), None)

        # There _is_ a default image
        userena_settings.USERENA_MUGSHOT_DEFAULT = "http://example.com"
        profile = Profile.objects.get(pk=1)
        self.assertEqual(profile.get_mugshot_url(), "http://example.com")

        # Settings back to default
        userena_settings.USERENA_MUGSHOT_GRAVATAR = True

    def test_get_mugshot_url_with_gravatar(self):
        """
        Test if the correct mugshot is returned when the user makes use of gravatar.

        """
        profile = Profile.objects.get(pk=1)

        gravatar_hash = hashlib.md5(profile.user.email.encode("utf-8")).hexdigest()

        # Test with the default settings
        mugshot_url = profile.get_mugshot_url()
        parsed = urlparse(mugshot_url)

        self.assertEqual(parsed.netloc, "www.gravatar.com")
        self.assertEqual(parsed.path, "/avatar/" + gravatar_hash)
        self.assertEqual(
            parse_qs(parsed.query),
            parse_qs(
                "s=%(size)s&d=%(default)s"
                % {
                    "size": userena_settings.USERENA_MUGSHOT_SIZE,
                    "default": userena_settings.USERENA_MUGSHOT_DEFAULT,
                }
            ),
        )

        # Change userena settings
        userena_settings.USERENA_MUGSHOT_SIZE = 180
        userena_settings.USERENA_MUGSHOT_DEFAULT = "404"

        # and test again
        mugshot_url = profile.get_mugshot_url()
        parsed = urlparse(mugshot_url)

        self.assertEqual(parsed.netloc, "www.gravatar.com")
        self.assertEqual(parsed.path, "/avatar/" + gravatar_hash)
        self.assertEqual(
            parse_qs(parsed.query),
            parse_qs(
                "s=%(size)s&d=%(default)s"
                % {
                    "size": userena_settings.USERENA_MUGSHOT_SIZE,
                    "default": userena_settings.USERENA_MUGSHOT_DEFAULT,
                }
            ),
        )

        # Settings back to default
        userena_settings.USERENA_MUGSHOT_SIZE = 80
        userena_settings.USERENA_MUGSHOT_DEFAULT = "identicon"

    def test_get_full_name_or_username(self):
        """ Test if the full name or username are returned correcly """
        user = User.objects.get(pk=1)
        profile = get_user_profile(user=user)

        # Profile #1 has a first and last name
        full_name = profile.get_full_name_or_username()
        self.assertEqual(full_name, "John Doe")

        # Let's empty out his name, now we should get his username
        user.first_name = ""
        user.last_name = ""
        user.save()

        self.assertEqual(profile.get_full_name_or_username(), "john")

        # Finally, userena doesn't use any usernames, so we should return the
        # e-mail address.
        userena_settings.USERENA_WITHOUT_USERNAMES = True
        self.assertEqual(profile.get_full_name_or_username(), "john@example.com")
        userena_settings.USERENA_WITHOUT_USERNAMES = False

    def test_can_view_profile(self):
        """ Test if the user can see the profile with three type of users. """
        anon_user = AnonymousUser()
        super_user = User.objects.get(pk=1)
        reg_user = User.objects.get(pk=2)

        profile = Profile.objects.get(pk=1)

        # All users should be able to see a ``open`` profile.
        profile.privacy = "open"
        self.assertTrue(profile.can_view_profile(anon_user))
        self.assertTrue(profile.can_view_profile(super_user))
        self.assertTrue(profile.can_view_profile(reg_user))

        # Registered and super users should be able to see a ``registered``
        # profile.
        profile.privacy = "registered"
        self.assertFalse(profile.can_view_profile(anon_user))
        self.assertTrue(profile.can_view_profile(super_user))
        self.assertTrue(profile.can_view_profile(reg_user))

        # Only superusers can see a closed profile.
        profile.privacy = "closed"
        self.assertFalse(profile.can_view_profile(anon_user))
        self.assertTrue(profile.can_view_profile(super_user))
        self.assertFalse(profile.can_view_profile(reg_user))
